<?php
namespace Importacao\Logic\IO;

use \Geral\Entities\Itemprojimport;
use Geral\Entities\Logprojimport;
use \Geral\Entities\Tabprojimport;
use Importacao\Logic\Integracao\ProjetoImportacao;

class UploadArquivo
{

	const EXTENSOES_ACEITAS = "xls|xlsx|ods";
    const EVENTOS           = "11";

	protected $nrorg;

	public function __construct($nrorg) 
	{
		$this->nrorg       = $nrorg;
	}

	public function salvaArquivo($caminhoArquivo, $tipoTemplate)
	{
		try {
			$tabprojimport = Tabprojimport::findOneByNrprojimportAndNrorg($tipoTemplate, 0);
            $newFileName = $this->getNewFileName($caminhoArquivo);
			$this->validaTransferencia($caminhoArquivo, $tabprojimport);
			$this->transfereArquivo($caminhoArquivo, $newFileName);
			
			$this->criaItemProjetoDeImportacao($tabprojimport, $newFileName);
		} catch (\Exception $e) {
			throw $e;
		}
	}

	protected function getNewFileName($caminhoArquivo)
	{
        $result = explode("/", $caminhoArquivo);
		return end($result);
	}

	private function transfereArquivo($caminhoArquivo, $nomeNovoArquivo)
	{
		$newDir = APP_PATH."/files/upload/".$this->nrorg."/data/";
		if (!file_exists($newDir)) {
			mkdir($newDir, 0777, true);
            chown($newDir, "root");
		}

        $result = explode("/", $caminhoArquivo);
		$oldDir     = UPLOAD_PATH;

		return rename($oldDir.end($result), "$newDir/$nomeNovoArquivo");
	}

	protected function validaTransferencia($arquivo, $tabprojimport)
	{
		if (file_exists($arquivo) === false) {
			throw new \Importacao\Logic\IO\Exception\ArquivoNaoEncontrado();
		}

		if (preg_match("/\.(".self::EXTENSOES_ACEITAS.")$/", $arquivo) === false) {
			throw new \Importacao\Logic\IO\Exception\ArquivoNaoSuportado();
		}

		if (($tabprojimport instanceof Tabprojimport) === false) {
			throw new \Importacao\Logic\IO\Exception\TipoTransferenciaNaoInformado();
		}
	}

	private function criaItemProjetoDeImportacao($tabprojimport, $newFileName)
	{
		try {

            $dataAtual      = \Geral\Util\DateUtil::getDataAtual();

			$itemprojimport = new Itemprojimport();
			$itemprojimport->setNrprojimport($tabprojimport->getNrprojimport());
			$itemprojimport->setNmarqimport($newFileName);
			$itemprojimport->setDtimportacao($dataAtual);
            $itemprojimport->setIdprojimport(Itemprojimport::STATUS_PENDENTE);
            $itemprojimport->setIdtpvalidacao(Itemprojimport::ETAPA_VALIDACAO);
            $itemprojimport->setIdarqimportado(Itemprojimport::ARQ_IMP_NAO);

			$itemprojimport->persist();
			\Setforms2\Shared\Registry::getEntityManager()->flush();

            $projetoImportacao = new ProjetoImportacao($itemprojimport, $this->nrorg);

            $nritemprojimport = $itemprojimport->getNritemprojimp();

            $this->criaLogDeItemProjeto($nritemprojimport);

		} catch (\Exception $e) {
			throw $e;
		}
	}

    private function criaLogDeItemProjeto($nritemprojimport)
    {
        try {

            $logprojimport = new Logprojimport();
            $logprojimport->setNritemprojimp($nritemprojimport);
            $logprojimport->setDscomandoimport(Logprojimport::MSG_SUCESSO_UPLOAD);
            $logprojimport->setNrtpvalidacao(Logprojimport::VALIDACAO_LAYOUT);

            $logprojimport->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function existePassoAnterior($nrprojimport)
    {
        try {
            self::validaDeParaEvento($nrprojimport);

            if ($nrprojimport != 1) {
                $resultado = Itemprojimport::findOneByNrprojimport($nrprojimport - 1);

                if (!is_object($resultado)) {
                    tk_message('O passo anterior não foi encontrado, favor realizar o passo '. ($nrprojimport - 1).".");
                    tk_set_visible_field('UPLOAD_ARQUIVO');
                } else {
                    tk_set_visible_field('UPLOAD_ARQUIVO');
                }
            } else {
                tk_set_visible_field('UPLOAD_ARQUIVO');
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    private static function validaDeParaEvento($nrprojimport)
    {
        if ($nrprojimport == self::EVENTOS) {
            self::dialogSave();
        }
    }

    /**
     *  Mensagem de atenção para o upload do arquivo de Eventos
     */
    public static function dialogSave() {

        $ok = array(
            "label"=> "Ok",
            "method"=> array(),
            "params_arr"=>array()
        );

        $msg = '<div style="padding:10px;">Prezado usuário, para importação do arquivo de "Eventos" é necessário realizar o DePara em relação aos eventos do HRM de acordo com a página 14 da especificação, caso tenha realizado este procedimento favor desconsiderar.</div>';
        tk_dialog($msg, array($ok));

    }

}